searchNodes=[{"type":"module","title":"Membrane.H265.Parser","doc":"Membrane element providing parser for H265 encoded video stream. The parser: prepares and sends the appropriate stream format, based on information provided in the stream and via the element's options splits the incoming stream into h265 access units - each buffer being output is a Membrane.Buffer struct with a binary payload of a single access unit enriches the output buffers with the metadata describing the way the access unit is split into NAL units, type of each NAL unit making up the access unit and the information if the access unit hold a keyframe. The parser works in one of three possible modes, depending on the structure of the input buffers: :bytestream - each input buffer contains some part of h265 stream's payload, but not necessary a logical h265 unit (like NAL unit or an access unit). Can be used for i.e. for parsing the stream read from the file. :nalu_aligned - each input buffer contains a single NAL unit's payload :au_aligned - each input buffer contains a single access unit's payload The parser's mode is set automatically, based on the input stream format received by that element: Receiving %Membrane.RemoteStream{type: :bytestream} results in the parser mode being set to :bytestream Receiving %Membrane.H265{alignment: :nalu} results in the parser mode being set to :nalu_aligned . Receiving %Membrane.H265{alignment: :au} results in the parser mode being set to :au_aligned . The parser also allows for conversion between stream structures. The available structures are: Annex B, :annexb - In a stream with this structure each NAL unit is prefixed by three or four-byte start code ( 0x(00)000001 ) that allows to identify boundaries between them. hvc1, :hvc1 - In such stream a DCR (Decoder Configuration Record) is included in stream_format and NALUs lack the start codes, but are prefixed with their length. The length of these prefixes is contained in the stream's DCR. PPSs, SPSs and VPSs (Picture Parameter Sets, Sequence Parameter Sets and Video Parameter Sets) are transported in the DCR. hev1, :hev1 - The same as hvc1, only that parameter sets may be also present in the stream (in-band). Element options Passed via struct Membrane.H265.Parser.t/0 vpss [ binary ( ) ] Default value: [] Video Parameter Set NAL unit binary payloads - if absent in the stream, may be provided via this option (only available for :annexb output stream format) Any decoder conforming to the profiles specified in &quot;Annex A&quot; of ITU/IEC H265 (08/21), but does not support INBLD may discard all VPS NAL units. spss [ binary ( ) ] Default value: [] Sequence Parameter Set NAL unit binary payloads - if absent in the stream, should be provided via this option (only available for :annexb output stream format). ppss [ binary ( ) ] Default value: [] Picture Parameter Set NAL unit binary payloads - if absent in the stream, should be provided via this option (only available for :annexb output stream format). skip_until_keyframe boolean ( ) Default value: true Determines whether to drop the stream until the first key frame is received. Defaults to true. repeat_parameter_sets boolean ( ) Default value: false Repeat all parameter sets ( vps , sps and pps ) on each IRAP picture. Parameter sets may be retrieved from: The stream Parser options. Decoder Configuration Record , sent in :hcv1 and :hev1 stream types output_alignment :au | :nalu Default value: :au Alignment of the buffers produced as an output of the parser. If set to :au , each output buffer will be a single access unit. Otherwise, if set to :nalu , each output buffer will be a single NAL unit. output_stream_structure nil | :annexb | :hvc1 | :hev1 | { :hvc1 | :hev1 , nalu_length_size :: pos_integer ( ) } Default value: nil format of the outgoing H265 stream, if set to :annexb NALUs will be separated by a start code (0x(00)000001) or if set to :hvc1 or :hev1 they will be prefixed by their size. Additionally for :hvc1 and :hev1 a tuple can be passed containing the atom and nalu_length_size that determines the size in bytes of each NALU's field describing their length (by default 4). In hvc1 output streams the PPSs, SPSs and VPSs will be transported in the DCR, when in hev1 they will be present only in the stream (in-band). If not provided or set to nil the stream's structure will remain unchanged. generate_best_effort_timestamps false | %{ :framerate =&gt; { pos_integer ( ) , pos_integer ( ) } , optional ( :add_dts_offset ) =&gt; boolean ( ) } Default value: false Generates timestamps based on given framerate . This option works only when Membrane.RemoteStream format arrives. Keep in mind that the generated timestamps may be inaccurate and lead to video getting out of sync with other media, therefore h265 stream should be kept in a container that stores the timestamps alongside. By default, the parser adds negative DTS offset to the timestamps, so that in case of frame reorder (which always happens when B frames are present) the DTS is always bigger than PTS. If that is not desired, you can set add_dts_offset: false . The calculated DTS/PTS may be wrong since we base it on access units' POC (Picture Order Count). We assume here that the POC is continuous on a CVS (Coded Video Sequence) which is not guaranteed by the H265 specification. An example where POC values may not be continuous is when generating sub-bitstream from the main stream by deleting access units belonging to a higher temporal sub-layer. Pads :input Accepted formats: % RemoteStream { type : :bytestream } H265 Direction: :input Availability: :always Flow control: :auto Demand unit: :buffers :output Accepted formats: % H265 { nalu_in_metadata? : true } Direction: :output Availability: :always Flow control: :auto Demand unit: nil","ref":"Membrane.H265.Parser.html"},{"type":"function","title":"Membrane.H265.Parser.options/0","doc":"Returns description of options available for this module","ref":"Membrane.H265.Parser.html#options/0"},{"type":"type","title":"Membrane.H265.Parser.stream_structure/0","doc":"Type referencing Membrane.H265.stream_structure type, in case of :hvc1 and :hev1 stream structure, it contains an information about the size of each NALU's prefix describing their length.","ref":"Membrane.H265.Parser.html#t:stream_structure/0"},{"type":"type","title":"Membrane.H265.Parser.t/0","doc":"Struct containing options for Membrane.H265.Parser","ref":"Membrane.H265.Parser.html#t:t/0"},{"type":"module","title":"Membrane.H265.Parser.AUSplitter","doc":"Module providing functionalities to group H265 NAL units into access units. The access unit splitter's behaviour is based on section 7.4.2.4.4 &quot;Order of NAL units and coded pictures and association to access units&quot; of the &quot;ITU-T Rec. H.265 (08/2021)&quot; specification.","ref":"Membrane.H265.Parser.AUSplitter.html"},{"type":"function","title":"Membrane.H265.Parser.AUSplitter.new/0","doc":"Returns a structure holding a clear state of the access unit splitter.","ref":"Membrane.H265.Parser.AUSplitter.html#new/0"},{"type":"function","title":"Membrane.H265.Parser.AUSplitter.split/3","doc":"Splits the given list of NAL units into the access units. It can be used for a stream which is not completely available at the time of function invocation, as the function updates the state of the access unit splitter - the function can be invoked once more, with new NAL units and the updated state. Under the hood, split/2 defines a finite state machine with two states: :first and :second . The state :first describes the state before reaching the first segment of a coded picture NALu of a given access unit. The state :second describes the state after processing the first segment of the coded picture of a given access unit.","ref":"Membrane.H265.Parser.AUSplitter.html#split/3"},{"type":"type","title":"Membrane.H265.Parser.AUSplitter.access_unit/0","doc":"A type representing an access unit - a list of logically associated NAL units.","ref":"Membrane.H265.Parser.AUSplitter.html#t:access_unit/0"},{"type":"opaque","title":"Membrane.H265.Parser.AUSplitter.t/0","doc":"A structure holding a state of the access unit splitter.","ref":"Membrane.H265.Parser.AUSplitter.html#t:t/0"},{"type":"module","title":"Membrane.H265.Parser.DecoderConfigurationRecord","doc":"Utility functions for parsing and generating HEVC Configuration Record. The structure of the record is described in section 8.3.3.1.1 of MPEG-4 part 15 (ISO/IEC 14496-15 Edition 2017-02).","ref":"Membrane.H265.Parser.DecoderConfigurationRecord.html"},{"type":"function","title":"Membrane.H265.Parser.DecoderConfigurationRecord.generate/4","doc":"Generates a DCR based on given PPSs, SPSs and VPSs.","ref":"Membrane.H265.Parser.DecoderConfigurationRecord.html#generate/4"},{"type":"function","title":"Membrane.H265.Parser.DecoderConfigurationRecord.parse/1","doc":"Parses the DCR.","ref":"Membrane.H265.Parser.DecoderConfigurationRecord.html#parse/1"},{"type":"type","title":"Membrane.H265.Parser.DecoderConfigurationRecord.t/0","doc":"Structure representing the Decoder Configuartion Record","ref":"Membrane.H265.Parser.DecoderConfigurationRecord.html#t:t/0"},{"type":"module","title":"Membrane.H265.Parser.Format","doc":"Module providing functionalities for preparing H265 format based on the parsed VPS and SPS NAL units.","ref":"Membrane.H265.Parser.Format.html"},{"type":"function","title":"Membrane.H265.Parser.Format.from_sps/3","doc":"Prepares the Membrane.H265.t() format based on the parsed SPS NALu. During the process, the function determines the profile of the h265 stream and the picture resolution.","ref":"Membrane.H265.Parser.Format.html#from_sps/3"},{"type":"module","title":"Membrane.H265.Parser.NALu","doc":"A module defining a struct representing a single NAL unit.","ref":"Membrane.H265.Parser.NALu.html"},{"type":"function","title":"Membrane.H265.Parser.NALu.int_type/1","doc":"","ref":"Membrane.H265.Parser.NALu.html#int_type/1"},{"type":"type","title":"Membrane.H265.Parser.NALu.t/0","doc":"A type defining the structure of a single NAL unit produced by the Membrane.H265.Parser.NALuParser . In the structure there is the following fields: parsed_fields - the map with keys being the NALu field names and the values being the value fetched from the NALu binary. They correspond to the NALu schemes defined in the section 7.3. of the &quot;ITU-T Rec. H.265 (08/2021)&quot; . prefix_length - number of bytes of the prefix used to split the NAL units in the bytestream. The prefix is defined as in: &quot;Annex B&quot; of the &quot;ITU-T Rec. H.265 (08/2021)&quot; . type - an atom representing the type of the NALu. Atom's name is based on the &quot;Table 7-1 – NAL unit type codes, syntax element categories, and NAL unit type classes&quot; of the &quot;ITU-T Rec. H.265 (08/2021)&quot; . payload - the binary, which parsing resulted in that structure being produced status - :valid , if the parsing was successfull, :error otherwise","ref":"Membrane.H265.Parser.NALu.html#t:t/0"},{"type":"type","title":"Membrane.H265.Parser.NALu.timestamps/0","doc":"","ref":"Membrane.H265.Parser.NALu.html#t:timestamps/0"},{"type":"module","title":"Membrane.H265.Parser.NALuParser","doc":"A module providing functionality of parsing a stream of binaries, out of which each is a payload of a single NAL unit.","ref":"Membrane.H265.Parser.NALuParser.html"},{"type":"function","title":"Membrane.H265.Parser.NALuParser.get_prefixed_nalu_payload/3","doc":"Returns payload of the NALu with appropriate prefix generated based on output stream structure and prefix length.","ref":"Membrane.H265.Parser.NALuParser.html#get_prefixed_nalu_payload/3"},{"type":"function","title":"Membrane.H265.Parser.NALuParser.new/1","doc":"Returns a structure holding a clear NALu parser state. input_stream_structure determines the prefixes of input NALU payloads.","ref":"Membrane.H265.Parser.NALuParser.html#new/1"},{"type":"function","title":"Membrane.H265.Parser.NALuParser.parse/4","doc":"Parses a binary representing a single NALu. Returns a structure that contains parsed fields fetched from that NALu. The input binary is expected to contain the prefix, defined as in the &quot;Annex B&quot; of the &quot;ITU-T Rec. H.265 (08/2021)&quot; .","ref":"Membrane.H265.Parser.NALuParser.html#parse/4"},{"type":"function","title":"Membrane.H265.Parser.NALuParser.parse_nalus/4","doc":"Parses a list of binaries, each representing a single NALu. See parse/3 for details.","ref":"Membrane.H265.Parser.NALuParser.html#parse_nalus/4"},{"type":"function","title":"Membrane.H265.Parser.NALuParser.prefix_nalus_payloads/2","doc":"","ref":"Membrane.H265.Parser.NALuParser.html#prefix_nalus_payloads/2"},{"type":"opaque","title":"Membrane.H265.Parser.NALuParser.t/0","doc":"A structure holding the state of the NALu parser.","ref":"Membrane.H265.Parser.NALuParser.html#t:t/0"},{"type":"module","title":"Membrane.H265.Parser.NALuSplitter","doc":"A module with functions responsible for splitting the h265 stream into the NAL units. The splitting is based on &quot;Annex B&quot; of the &quot;ITU-T Rec. H.265 (08/2021)&quot; .","ref":"Membrane.H265.Parser.NALuSplitter.html"},{"type":"function","title":"Membrane.H265.Parser.NALuSplitter.new/2","doc":"Returns a structure holding a NALu splitter state. By default, the inner unparsed_payload of the state is clean. However, there is a possibility to set that unparsed_payload to a given binary, provided as an argument of the new/1 function.","ref":"Membrane.H265.Parser.NALuSplitter.html#new/2"},{"type":"function","title":"Membrane.H265.Parser.NALuSplitter.split/3","doc":"Splits the binary into NALus sequence. Takes a binary h265 stream as an input and produces a list of binaries, where each binary is a complete NALu that can be passed to the Membrane.H265.Parser.NALuParser.parse/2 .","ref":"Membrane.H265.Parser.NALuSplitter.html#split/3"},{"type":"opaque","title":"Membrane.H265.Parser.NALuSplitter.t/0","doc":"A structure holding the state of the NALu splitter.","ref":"Membrane.H265.Parser.NALuSplitter.html#t:t/0"},{"type":"module","title":"Membrane.H265.Parser.NALuTypes","doc":"The module aggregating the mapping of nal_unit_type fields of the NAL unit to the human-friendly name of a NALu type.","ref":"Membrane.H265.Parser.NALuTypes.html"},{"type":"function","title":"Membrane.H265.Parser.NALuTypes.get_type/1","doc":"The function which returns the human friendly name of a NALu type for a given nal_unit_type . The mapping is based on: &quot;Table 7-1 – NAL unit type codes, syntax element categories, and NAL unit type classes&quot; of the &quot;ITU-T Rec. H.265 (08/2021)&quot;","ref":"Membrane.H265.Parser.NALuTypes.html#get_type/1"},{"type":"function","title":"Membrane.H265.Parser.NALuTypes.irap_nalus/0","doc":"Returns the list of IRAP NAL units. An IRAP (Intra Random Access Picture) NALu type denotes a picture that can be decoded without referencing prior pictures. IDR and BLA NALus start a new VCS(Video Coded Sequence).","ref":"Membrane.H265.Parser.NALuTypes.html#irap_nalus/0"},{"type":"function","title":"Membrane.H265.Parser.NALuTypes.vcl_nalu_types/0","doc":"Get all the VCL NAL units","ref":"Membrane.H265.Parser.NALuTypes.html#vcl_nalu_types/0"},{"type":"type","title":"Membrane.H265.Parser.NALuTypes.nalu_type/0","doc":"A type representing all the possible human-friendly names of NAL unit types.","ref":"Membrane.H265.Parser.NALuTypes.html#t:nalu_type/0"},{"type":"extras","title":"Membrane H265 Plugin","doc":"Membrane H265 parser. It is the Membrane element responsible for parsing the incoming h265 stream. The parsing is done as a sequence of the following steps: Splitting the h265 stream into stream NAL units, based on the &quot;Annex B&quot; of the &quot;ITU-T Rec. H.265 (08/2021)&quot; Parsing the NAL unit headers, so that to read the type of the NAL unit Parsing the NAL unit body with the appropriate scheme, based on the NAL unit type read in the step before Aggregating the NAL units into a stream of access units The output of the element is the incoming binary payload, enriched with the metadata describing the division of the payload into access units.","ref":"readme.html"},{"type":"extras","title":"Membrane H265 Plugin - Installation","doc":"The package can be installed by adding membrane_h265_plugin to your list of dependencies in mix.exs : def deps do [ { :membrane_h265_plugin , &quot;~&gt; 0.3.0&quot; } ] end","ref":"readme.html#installation"},{"type":"extras","title":"Membrane H265 Plugin - Usage","doc":"The following pipeline takes H265 file, parses it, and then decodes it to the raw video. defmodule Decoding.Pipeline do use Membrane.Pipeline alias Membrane . { File , H265 } @impl true def handle_init ( _ctx , _opts ) do structure = child ( :source , % File.Source { location : &quot;test/fixtures/input-10-1920x1080.h265&quot; } ) |&gt; child ( :parser , H265.Parser ) |&gt; child ( :decoder , H265.FFmpeg.Decoder ) |&gt; child ( :sink , % File.Sink { location : &quot;output.raw&quot; } ) { [ spec : structure ] , nil } end @impl true def handle_element_end_of_stream ( :sink , _ctx_ , state ) do { [ terminate : :normal ] , state } end end","ref":"readme.html#usage"},{"type":"extras","title":"LICENSE","doc":"Apache License Version 2.0, January 2004 http://www.apache.org/licenses/ TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION 1. Definitions. &quot;License&quot; shall mean the terms and conditions for use, reproduction, and distribution as defined by Sections 1 through 9 of this document. &quot;Licensor&quot; shall mean the copyright owner or entity authorized by the copyright owner that is granting the License. &quot;Legal Entity&quot; shall mean the union of the acting entity and all other entities that control, are controlled by, or are under common control with that entity. For the purposes of this definition, &quot;control&quot; means (i) the power, direct or indirect, to cause the direction or management of such entity, whether by contract or otherwise, or (ii) ownership of fifty percent (50%) or more of the outstanding shares, or (iii) beneficial ownership of such entity. &quot;You&quot; (or &quot;Your&quot;) shall mean an individual or Legal Entity exercising permissions granted by this License. &quot;Source&quot; form shall mean the preferred form for making modifications, including but not limited to software source code, documentation source, and configuration files. &quot;Object&quot; form shall mean any form resulting from mechanical transformation or translation of a Source form, including but not limited to compiled object code, generated documentation, and conversions to other media types. &quot;Work&quot; shall mean the work of authorship, whether in Source or Object form, made available under the License, as indicated by a copyright notice that is included in or attached to the work (an example is provided in the Appendix below). &quot;Derivative Works&quot; shall mean any work, whether in Source or Object form, that is based on (or derived from) the Work and for which the editorial revisions, annotations, elaborations, or other modifications represent, as a whole, an original work of authorship. For the purposes of this License, Derivative Works shall not include works that remain separable from, or merely link (or bind by name) to the interfaces of, the Work and Derivative Works thereof. &quot;Contribution&quot; shall mean any work of authorship, including the original version of the Work and any modifications or additions to that Work or Derivative Works thereof, that is intentionally submitted to Licensor for inclusion in the Work by the copyright owner or by an individual or Legal Entity authorized to submit on behalf of the copyright owner. For the purposes of this definition, &quot;submitted&quot; means any form of electronic, verbal, or written communication sent to the Licensor or its representatives, including but not limited to communication on electronic mailing lists, source code control systems, and issue tracking systems that are managed by, or on behalf of, the Licensor for the purpose of discussing and improving the Work, but excluding communication that is conspicuously marked or otherwise designated in writing by the copyright owner as &quot;Not a Contribution.&quot; &quot;Contributor&quot; shall mean Licensor and any individual or Legal Entity on behalf of whom a Contribution has been received by Licensor and subsequently incorporated within the Work. 2. Grant of Copyright License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable copyright license to reproduce, prepare Derivative Works of, publicly display, publicly perform, sublicense, and distribute the Work and such Derivative Works in Source or Object form. 3. Grant of Patent License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable (except as stated in this section) patent license to make, have made, use, offer to sell, sell, import, and otherwise transfer the Work, where such license applies only to those patent claims licensable by such Contributor that are necessarily infringed by their Contribution(s) alone or by combination of their Contribution(s) with the Work to which such Contribution(s) was submitted. If You institute patent litigation against any entity (including a cross-claim or counterclaim in a lawsuit) alleging that the Work or a Contribution incorporated within the Work constitutes direct or contributory patent infringement, then any patent licenses granted to You under this License for that Work shall terminate as of the date such litigation is filed. 4. Redistribution. You may reproduce and distribute copies of the Work or Derivative Works thereof in any medium, with or without modifications, and in Source or Object form, provided that You meet the following conditions: (a) You must give any other recipients of the Work or Derivative Works a copy of this License; and (b) You must cause any modified files to carry prominent notices stating that You changed the files; and (c) You must retain, in the Source form of any Derivative Works that You distribute, all copyright, patent, trademark, and attribution notices from the Source form of the Work, excluding those notices that do not pertain to any part of the Derivative Works; and (d) If the Work includes a &quot;NOTICE&quot; text file as part of its distribution, then any Derivative Works that You distribute must include a readable copy of the attribution notices contained within such NOTICE file, excluding those notices that do not pertain to any part of the Derivative Works, in at least one of the following places: within a NOTICE text file distributed as part of the Derivative Works; within the Source form or documentation, if provided along with the Derivative Works; or, within a display generated by the Derivative Works, if and wherever such third-party notices normally appear. The contents of the NOTICE file are for informational purposes only and do not modify the License. You may add Your own attribution notices within Derivative Works that You distribute, alongside or as an addendum to the NOTICE text from the Work, provided that such additional attribution notices cannot be construed as modifying the License. You may add Your own copyright statement to Your modifications and may provide additional or different license terms and conditions for use, reproduction, or distribution of Your modifications, or for any such Derivative Works as a whole, provided Your use, reproduction, and distribution of the Work otherwise complies with the conditions stated in this License. 5. Submission of Contributions. Unless You explicitly state otherwise, any Contribution intentionally submitted for inclusion in the Work by You to the Licensor shall be under the terms and conditions of this License, without any additional terms or conditions. Notwithstanding the above, nothing herein shall supersede or modify the terms of any separate license agreement you may have executed with Licensor regarding such Contributions. 6. Trademarks. This License does not grant permission to use the trade names, trademarks, service marks, or product names of the Licensor, except as required for reasonable and customary use in describing the origin of the Work and reproducing the content of the NOTICE file. 7. Disclaimer of Warranty. Unless required by applicable law or agreed to in writing, Licensor provides the Work (and each Contributor provides its Contributions) on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied, including, without limitation, any warranties or conditions of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. You are solely responsible for determining the appropriateness of using or redistributing the Work and assume any risks associated with Your exercise of permissions under this License. 8. Limitation of Liability. In no event and under no legal theory, whether in tort (including negligence), contract, or otherwise, unless required by applicable law (such as deliberate and grossly negligent acts) or agreed to in writing, shall any Contributor be liable to You for damages, including any direct, indirect, special, incidental, or consequential damages of any character arising as a result of this License or out of the use or inability to use the Work (including but not limited to damages for loss of goodwill, work stoppage, computer failure or malfunction, or any and all other commercial damages or losses), even if such Contributor has been advised of the possibility of such damages. 9. Accepting Warranty or Additional Liability. While redistributing the Work or Derivative Works thereof, You may choose to offer, and charge a fee for, acceptance of support, warranty, indemnity, or other liability obligations and/or rights consistent with this License. However, in accepting such obligations, You may act only on Your own behalf and on Your sole responsibility, not on behalf of any other Contributor, and only if You agree to indemnify, defend, and hold each Contributor harmless for any liability incurred by, or claims asserted against, such Contributor by reason of your accepting any such warranty or additional liability. END OF TERMS AND CONDITIONS APPENDIX: How to apply the Apache License to your work. To apply the Apache License to your work, attach the following boilerplate notice, with the fields enclosed by brackets &quot;[]&quot; replaced with your own identifying information. (Don't include the brackets!) The text should be enclosed in the appropriate comment syntax for the file format. We also recommend that a file or class name and description of purpose be included on the same &quot;printed page&quot; as the copyright notice for easier identification within third-party archives. Copyright 2022 Software Mansion Licensed under the Apache License, Version 2.0 (the &quot;License&quot;); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.","ref":"license.html"}]